@@ -1,23 +1,35 @@
 from datetime import datetime
 from enum import Enum
+from io import BytesIO
 from pathlib import Path
 from typing import (
     Any,
+    Dict,
+    Iterator,
     List,
     Optional,
     TYPE_CHECKING,
     Union,
 )
+from zipfile import ZipFile
 
 from pydantic import (
     AnyHttpUrl,
     Field,
-    validator,
 )
+from typing_extensions import Literal
 from yarl import URL
 
-from async_pixiv.model._base import PixivModel
-from async_pixiv.model.other import ImageUrl
+from async_pixiv.error import ArtWorkTypeError
+from async_pixiv.model._base import (
+    PixivModel,
+    null_dict_validator,
+)
+from async_pixiv.model.other import (
+    ImageUrl,
+    Series,
+    Tag,
+)
 from async_pixiv.model.user import User
 
 if TYPE_CHECKING:
@@ -29,22 +41,12 @@ if TYPE_CHECKING:
     )
 
 __all__ = [
-    'Tag', 'TagTranslation',
     'ArtWork', 'ArtWorkType',
     'Comment',
+    'UgoiraMetadata',
 ]
 
 
-class TagTranslation(PixivModel):
-    zh: Optional[str]
-    en: Optional[str]
-
-
-class Tag(PixivModel):
-    name: str
-    translated_name: Optional[str]
-
-
 class ArtWorkType(Enum):
     illust = 'illust'
     ugoira = 'ugoira'
@@ -61,12 +63,8 @@ class ArtWorkType(Enum):
                 return False
 
 
-# noinspection PyProtectedMember
+# noinspection PyProtectedMember,PyShadowingBuiltins
 class ArtWork(PixivModel):
-    class Series(PixivModel):
-        id: int
-        title: str
-
     class MetaPage(PixivModel):
         image_urls: "ImageUrl"
 
@@ -185,6 +183,52 @@ class ArtWork(PixivModel):
                 ), output=output))
             return result
 
+    async def ugoira_metadata(
+            self, client: Optional["PixivClient"] = None
+    ) -> Optional["UgoiraMetadata"]:
+        if self.type != ArtWorkType.ugoira:
+            return None
+        if client is None:
+            from async_pixiv.client import PixivClient
+            client = PixivClient.get_client()
+        return (await client.ILLUST.ugoira_metadata(self.id)).metadata
+
+    async def download_ugoira(
+            self, client: Optional["PixivClient"] = None, *,
+            type: Literal['zip', 'jpg', 'all', 'iter'] = 'zip'
+    ) -> Optional[Union[
+        ZipFile, List[bytes], Dict[str, Union[ZipFile, List[bytes]]],
+        Iterator[bytes]
+    ]]:
+        if self.type != ArtWorkType.ugoira:
+            raise ArtWorkTypeError(
+                'If you want to download a moving image, '
+                'please use this method: \"download\"'
+            )
+        if client is None:
+            from async_pixiv.client import PixivClient
+            client = PixivClient.get_client()
+        metadata = await self.ugoira_metadata(client)
+        zip_url = metadata.zip_url
+        data = await client.download(
+            zip_url.original or zip_url.large or zip_url.medium,
+        )
+        if data is None:
+            return None
+        zip_file = ZipFile(BytesIO(data))
+        if type == 'zip':
+            return zip_file
+        frames = []
+        for frame_info in metadata.frames:
+            with zip_file.open(frame_info.file) as f:
+                frames.append(f.read())
+        if type == 'jpg':
+            return frames
+        elif type == 'iter':
+            return iter(frames)
+        else:
+            return {'zip': zip_file, 'frames': frames}
+
 
 class Comment(PixivModel):
     id: int
@@ -193,9 +237,13 @@ class Comment(PixivModel):
     user: User
     parent: Optional["Comment"] = Field(alias='parent_comment')
 
-    @validator('parent', pre=True, always=True)
-    def parent_check(cls, value):
-        if value == {}:
-            return None
-        else:
-            return value
+    _check = null_dict_validator('parent')
+
+
+class UgoiraMetadata(PixivModel):
+    class Frame(PixivModel):
+        file: str
+        delay: int
+
+    zip_url: ImageUrl = Field(alias='zip_urls')
+    frames: List[Frame]
